# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import numpy as np
from . import interpolate_by_finite_diff


def interpolate_ahead(x, y, n, direction="fw"):
    if not np.all(np.diff(x) > 0):
        raise ValueError("x not strictly monotonic.")

    if direction == "both":
        y1, slc1 = interpolate_ahead(x, y, n, "fw")
        y2, slc2 = interpolate_ahead(x, y, n, "bw")
        y = np.zeros_like(x)
        y[slc1] = y1
        y[slc2] += y2
        y[slc1.start : slc2.stop] /= 2
        return y, slice(None, None)
    elif direction == "fw":
        forward = True
    elif direction == "bw":
        forward = False
    else:
        raise ValueError("Unknown direction: %s" % direction)

    rev = slice(None, None, 1) if forward else slice(None, None, -1)
    values = []
    for idx, (xv, yv) in enumerate(zip(x[rev][n:], y[rev][n:])):
        _x = np.ascontiguousarray(x[rev][idx : idx + n], dtype=np.float64)
        _y = np.ascontiguousarray(y[rev][idx : idx + n], dtype=np.float64)
        _v = np.array([xv])
        values.append(
            interpolate_by_finite_diff(_x, _y, _v, maxorder=0, ntail=n, nhead=0)
        )
    return np.array(values[rev]).squeeze(), slice(n, None) if forward else slice(
        None, -n
    )


def avg_stddev(arr, w):
    """Calculates the average and standard deviation.

    Parameters
    ----------
    arr : array_like
        Values.
    w : array_like
        Weights.

    Returns
    -------
    tuple of 2 floats (average & standard deviation)

    """
    avg, wsum = np.average(arr, weights=w, returned=True)
    res = arr - avg
    stddev = np.sqrt(np.sum(np.dot(w, np.square(res)) / (res.size - 1) / wsum))
    return avg, stddev



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import math
import numpy as np


def sigm(x, m, o, n):
    return m * (x - o) * ((m * (x - o)) ** n + 1) ** -(1.0 / n)


def g(x):
    s = sigm(x, 20, 1, 4)
    s2 = sigm(x, 5, 1, 4)
    g.nelem_call += x.size
    g.nfev += 1
    return 10 * (1 - s2 ** 2) * np.exp(20 * s) / np.exp(20 * x) + 0.1 * (x - 3) ** 2


g.nelem_call = 0
g.nfev = 0


def g2(x):
    return g(2 * x) + g(x)


g3 = np.vectorize(
    lambda t: (math.exp(-t) + (0.2 if round(t * 1234567) % 3 == 0 else 0))
    if t < 5
    else 0.5 * math.exp(5 - t)
)  # noisy function



# -*- coding: utf-8 -*-
"""
Finite difference weights for any derivative order on arbitrarily spaced grids.
"""
from __future__ import absolute_import, division, print_function

from ._release import __version__

from ._finitediff_c import (
    derivatives_at_point_by_finite_diff,
    interpolate_by_finite_diff,
    get_weights,
)

__all__ = [
    "derivatives_at_point_by_finite_diff",
    "interpolate_by_finite_diff",
    "get_weights",
]


def get_include():
    from pkg_resources import resource_filename, Requirement

    return resource_filename(Requirement.parse(__name__), "%s/include" % __name__)



# -*- coding: utf-8 -*-
# distutils: sources = ['finitediff/external/newton_interval/src/newton_interval.c', 'src/finitediff_c.c']
# cython: language_level=3

cimport numpy as cnp
import numpy as np

from newton_interval cimport get_interval, get_interval_from_guess
from finitediff_c cimport finitediff_calc_and_apply_fd, finitediff_calculate_weights, finitediff_interpolate_by_finite_diff


def get_weights(grid, double xtgt, int n=-1, int maxorder=0):
    """
    Generates finite differnece weights.

    Parameters
    ----------
    grid: array_like
        Grid points.
    xtgt: float
        Point at which estimates should be accurate.
    n: int, optional
        Number of points used in ``xarr``. default: -1 (means use length of xarr).
    maxorder: int, optional
        default: 0 (means interpolation)

    Returns
    -------
    array_like
         2 dimensional array with shape==(n, maxorder+1) with
         Fortran order (contiguous along columns)
         with weights for 0:th order in first column.
    """
    cdef cnp.ndarray[cnp.float64_t, ndim=1] xarr = np.ascontiguousarray(np.ravel(grid), dtype=np.float64)
    if n == -1:
        n = xarr.size
    cdef cnp.ndarray[cnp.float64_t, ndim=2, mode='fortran'] c = \
        np.empty((n, maxorder+1), order='F')
    finitediff_calculate_weights(&c[0, 0], n, &xarr[0], n, maxorder, xtgt)
    return c


def derivatives_at_point_by_finite_diff(
        grid, ydata, double xtgt,
        int maxorder, yorder='C', reshape=None):
    """ Esimates of derivatives up to specified order at a point.

    Estimates derivatives/function values of requested order
    at multiple points (xtgt) based on finite difference using
    provided ``grid`` and ``ydata``.

    Parameters
    ----------
    grid : array_like
        Grid points: values of the independent variable ("x-data").
    ydata : array_like
        Values of the dependent variable. May be two dimensional, in
        which case the weights of the grid is reused.
    xtgt : float
        The target value of the independent variable where the
        the finite difference scheme should be applied.
    maxorder : int, optional
        Maximum order of derivatives to estimate.
        The default is 0 (interpolation).
    yorder : char
        NumPy "order" of ydata.
    reshape: bool
        Whether to return a 2D array or not. Default:
        if ``ydata.ndim != 1``.


    Returns
    -------
    numpy.ndarray
        Estimate from applying the finite difference scheme.

    Examples
    --------
    >>> derivatives_at_point_by_finite_diff(np.array([.0, .5, 1.]),
            np.array([.0, .25, 1.]), .5, 2)  # y=x**2
    array([.25, 1.0, 2.0])  # (x**2, 2x, 2)

    References
    ----------
    The underlying algorithm is from:
    Generation of Finite Difference Formulas on Arbitrarily Spaced Grids,
    Bengt Fornberg, Mathematics of compuation, 51, 184, 1988, 699-706
    """
    ydata = np.asarray(ydata)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] xarr = np.ascontiguousarray(grid, dtype=np.float64)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] yarr = np.ascontiguousarray(np.ravel(ydata, order=yorder), dtype=np.float64)
    if yarr.size % xarr.size:
        raise ValueError("Incompatible shapes: grid & ydata")
    cdef int nsets = yarr.size // xarr.size
    cdef cnp.ndarray[cnp.float64_t, ndim=1] yout = np.empty((maxorder+1)*nsets)
    if xarr.size < maxorder+1:
        raise ValueError("xdata too short for requested derivative order")
    finitediff_calc_and_apply_fd(&yout[0], xarr.size, nsets, maxorder, xarr.size, &xarr[0], &yarr[0], xarr.size, xtgt)
    if reshape is None:
        reshape = ydata.ndim != 1
    if reshape:
        return yout.reshape((nsets, maxorder+1))
    else:
        return yout

def interpolate_by_finite_diff(
        grid, ydata, xtgts, int maxorder=0, int ntail=2, int nhead=2, yorder='C', reshape=None):
    """ Estimates derivatives of requested order at multiple points.

    Estimates derivatives/function values of requested order
    at multiple points (``xtgts``) based on finite difference using
    provided ``grid`` and ``ydata``.

    Parameters
    ----------
    grid : array_like
        Values of the independent variable ("x-data").
    ydata : array_like
        Values of the dependent variable.
    xtgts : array_like
        Values of the independent variable where the
        the finite difference scheme should be applied.
    maxorder : int, optional
        Up to what order derivatives are to be estimated.
        The default is 0 (interpolation).
    ntail : int, optional
        how many points in ``grid`` before ``xtgts`` to inclued (default = 2).
    nhead : int, optional
        how many points in ``grid`` after ``xtgts`` to include (default = 2).
    yorder : char
        NumPy "order" of ydata.
    reshape: bool
        Whether to return a 3D array or not. Default:
        if ``ydata.ndim != 1``.

    Returns
    -------
    array_like
        Estimates from applying the finite difference scheme

    Examples
    --------
    >>> import numpy as np
    >>> from finitediff import interpolate_by_finite_diff as ifd
    >>> x = np.array([0, 1, 2])
    >>> y = np.array([[2, 3, 5], [3, 4, 7], [7, 8, 9], [3, 4, 6]])
    >>> xout = np.linspace(0.5, 1.5, 5)
    >>> r = ifd(x, y, xout, maxorder=2)
    >>> r.shape
    (5, 4, 3)

    Notes
    -----
    It is required that: ``order >= ntail + nhead``
    Algortithm assumes non-regularly spaced ``grid``. If
    ``grid`` is regularly spaced this algortihm is not optimal
    from a performance perspective.

    References
    ----------
    The underlying algorithm is from:
    Generation of Finite Difference Formulas on Arbitrarily Spaced Grids,
    Bengt Fornberg, Mathematics of computation, 51, 184, 1988, 699-706
    """
    ydata = np.asarray(ydata)
    xtgts = np.asarray(xtgts)
    cdef:
        int flag
        int nin = ntail+nhead
        int nout = xtgts.size
        cnp.ndarray[cnp.float64_t, ndim=1] xgrd = np.ascontiguousarray(grid, dtype=np.float64)
        cnp.ndarray[cnp.float64_t, ndim=1] tgts = np.ascontiguousarray(xtgts, dtype=np.float64)
        cnp.ndarray[cnp.float64_t, ndim=1] yarr = np.ascontiguousarray(np.ravel(ydata, order=yorder), dtype=np.float64)
        int nsets = yarr.size // xgrd.size
        cnp.ndarray[cnp.float64_t, ndim=1] yout = np.zeros(
            (nout*nsets*(maxorder+1)), order='C', dtype=np.float64)

    if yarr.size % xgrd.size:
        raise ValueError("Incompatible shapes: grid & ydata")

    flag = finitediff_interpolate_by_finite_diff(
        <double*>yout.data, nout, nsets, maxorder, nsets*(maxorder+1), maxorder+1,
        ntail, nhead, <double*>xgrd.data, xgrd.size, <double*>yarr.data, xgrd.size,
        <double*>tgts.data
    )
    if flag == 1:
        raise ValueError("Bad alloc")
    elif flag == 2:
        raise ValueError("grid is too small")
    if flag == 4:
        raise ValueError("too few points")

    if reshape is None:
        reshape = ydata.ndim != 1
    if reshape:
        return yout.reshape((nout, nsets, maxorder+1))
    else:
        return yout.reshape((nout, -1))



# Cython pxd file (http://docs.cython.org/src/tutorial/pxd_files.html)
# for c_fornberg.f90 (Fortran 2003 module wrapping fornberg.f90 using ISO_C_BINDING)
cdef extern void fornberg_apply_fd(int nin, int maxorder,
    const double * const xdata, const double * const ydata,
    double xtgt, double * const out)

cdef extern void fornberg_populate_weights(double z, const double * const x,
    int nd, int m, double * const c)




# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import numpy as np
import pytest

from ..rebalance import pre_pruning_mask


def test_pre_pruning_mask():
    assert np.all(pre_pruning_mask(np.array([0.0, 1e00, 2e00, 3, 4])) == [True] * 5)
    assert np.all(
        pre_pruning_mask(np.array([0.0, 1e-14, 2e00, 3, 4]), atol=1e-12)
        == [True, False, True, True, True]
    )
    assert np.all(
        pre_pruning_mask(np.array([0.0, 1e-14, 2e-14, 3, 4]), atol=1e-12)
        == [True, False, False, True, True]
    )
    assert np.all(
        pre_pruning_mask(np.array([0.0, 1e00, 2e00, 4, 4 + 2e-12]))
        == [True, True, True, False, True]
    )
    assert np.all(
        pre_pruning_mask(np.array([0.0, 1e00, 4, 4 + 1e-12, 4 + 2e-12]))
        == [True, True, False, False, True]
    )
    assert np.all(
        pre_pruning_mask(np.array([0.0, 1e-14, 2e00, 4, 4 + 2e-12]), atol=1e-12)
        == [True, False, True, False, True]
    )

    with pytest.raises(ValueError):
        assert pre_pruning_mask(np.array([1.0, 1 + 1e-13, 1 + 2e-13]))



# -*- coding: utf-8; mode: cython -*-

cdef extern from "finitediff_templated.hpp" namespace "finitediff":
     cdef void populate_weights[T](const T, const T * const,
                                   const int, const int,
                                   T * const)
     cdef void apply_fd[T](const int, const int,
                           const T * const, const T * const,
                           const T, T * const)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import math
import numpy as np
from scipy.interpolate import interp1d


def _avgdiff(x):
    dx = np.diff(x)
    dx2 = np.zeros_like(x)
    dx2[0], dx2[-1] = dx[0], dx[-1]
    dx2[1:-1] = 0.5 * (dx[1:] + dx[:-1])
    return dx2


def rebalanced_grid(
    grid, err, base=0.25, num=None, resolution_factor=10, smooth_fact=1.0
):
    if num is None:
        num = grid.size

    dx = np.diff(grid)
    area_err = 0.5 * np.dot(err[1:] + err[:-1], dx)  # trapezoidal rule
    dx2 = _avgdiff(grid)

    def smooth_err(x):
        tot = 0
        for i, (gx, e) in enumerate(zip(grid, err)):
            fwhm = dx2[i] * smooth_fact
            tot += e * np.exp(-((x - gx) ** 2) / (2 * (fwhm / 2.35482) ** 2))
        return tot

    finegrid = np.zeros((grid.size - 1) * resolution_factor + 1)
    for i in range(grid.size - 1):
        finegrid[i * resolution_factor : (i + 1) * resolution_factor] = np.linspace(
            grid[i], grid[i + 1], resolution_factor + 1
        )[:-1]
    finegrid[-resolution_factor - 1 :] = np.linspace(
        grid[-2], grid[-1], resolution_factor + 1
    )
    smoothed = smooth_err(finegrid) + base * area_err / (grid[-1] - grid[0])
    assert np.all(smoothed > 0)
    assert np.all(_avgdiff(finegrid) > 0)
    interr = np.cumsum(smoothed * _avgdiff(finegrid))
    cb = interp1d(interr, finegrid)
    return cb(np.linspace(interr[0], interr[-1], num))


def pre_pruning_mask(grid, rtol=1e-12, atol=0.0):
    """Returns a mask for grid pruning.

    Any grid spacing smaller than ``rtol*gridvalue + atol`` will
    be pruned. In general the value on the right is removed unless it is
    the last point in the grid.

    Parameters
    ----------
    grid : array
    rtol : float
    atol : float

    Returns
    -------
    NumPy array of ``numpy.bool_`` (to be used as mask).

    """
    if np.any(np.diff(grid) < 0):
        raise ValueError("grid needs to be monotonic")
    limit = grid[-1] - (atol + abs(rtol * grid[-1]))
    mask = np.empty(grid.size, dtype=np.bool_)
    mask[grid.size - 1] = True  # rightmost point included
    for ridx in range(grid.size - 2, -1, -1):
        if grid[ridx] < limit:
            mask[ridx] = True
            break
        else:
            mask[ridx] = False
    else:
        raise ValueError("no grid-points left")
    mask[0] = True  # leftmost point included
    limit = grid[0] + abs(rtol * grid[0]) + atol
    for idx in range(1, ridx):
        if grid[idx] < limit:
            mask[idx] = False
        else:
            mask[idx] = True
            limit = grid[idx] + abs(rtol * grid[idx]) + atol
    return mask


def combine_grids(grids, **kwargs):
    """Combines multiple grids and prunes them using pre_pruning mask

    Parameters
    ----------
    grids : iterable of array_like grids
    \\*\\* : dict
        Keyword arguments passed on to pre_pruning_mask

    Returns
    -------
    Strictly increasing monotonic array

    """
    supergrid = np.sort(np.concatenate(grids))
    mask = pre_pruning_mask(supergrid, **kwargs)
    return supergrid[mask]


def grid_pruning_mask(grid, err, ndrop=None, protect_sparse=None, pow_err=2, pow_dx=2):
    """Returns a mask for grid pruning.

    Parameters
    ----------
    grid : array
    err : array
    ndrop : int
        If not provided taken as 25% of grid size (rounded upward).
    protect_sparse : int
        If not provided taken as 25% of grid size (rounded upward).
    pow_err : number
        Exponent of error in weighting.
    pow_dx : number
        Exponent of grid spacing in weighting.

    """
    if ndrop is None:
        ndrop = math.ceil(grid.size * 0.25)
    if protect_sparse is None:
        protect_sparse = math.ceil(grid.size * 0.25)
    dx = _avgdiff(grid)
    protected = np.argsort(dx)[-protect_sparse:]
    score = err ** pow_err * dx ** pow_dx
    importance = np.argsort(score)
    drop = []
    for considered in importance:
        if considered in protected:
            continue
        if considered - 1 in drop or considered + 1 in drop:
            continue
        drop.append(considered)
        if len(drop) == ndrop:
            break
    return ~np.in1d(np.arange(grid.size), drop)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import numpy as np

from ..util import interpolate_ahead


def refine_grid(
    grid,
    cb,
    grid_additions=(50, 50),
    ntrail=2,
    blurs=((), ()),
    metric=None,
    atol=None,
    rtol=None,
    extremum_refinement=None,
    snr=False,
):
    """Refines an existing grid by adding points to it.

    Parameters
    ----------
    grid : array
    cb : callbable
        Function to be evaluated (note that noise is handled poorly).
    grid_additions : iterable of ints (even numbers)
        Sequence specifying how many gridpoints to add each time.
    ntrail : int (>= 2)
        Number of points to include in the look-ahead extrapolation.
    blurs : pair of iterables of ints (of same length)
        Blur fractions of absolute residuals to neighbors.
    atol : float
        Absolute tolerance to be fulfilled by all absolute residuals for early exit.
    rtol : float
        Relative tolerance to be fulfilled by all absolute residuals for early exit.
    extremum_refinement : locator (callable), n (int), predicate (callable)
        Between each grid addition a callable for locating the extremum (e.g. np.argmax)
        can be evaluated. The integer specifies how many gridpoints that should be inserted
        on each side (one side if on boundary) of the extremum.
    snr : bool
        Use signal-to-noise ratio the lower the grid-addition-weight of potential noise.

    Returns
    -------
    (grid, errors)

    """
    for na in grid_additions:
        if (na % 2) != 0:
            raise ValueError("Need even number of grid points for each addition")
    if extremum_refinement == "max":
        extremum_refinement = (np.argmax, 1, lambda y, i: True)
    elif extremum_refinement == "min":
        extremum_refinement = (np.argmin, 1, lambda y, i: True)

    def add_to(adds, grd, res, ys):
        na = np.sum(adds)
        if na == 0:
            return grd, res, ys
        nextresults = np.empty(grd.size + na, dtype=object)
        nextresults[0] = res[0]
        nexty = np.empty(grd.size + na)
        nexty[0] = ys[0]
        nextgrid = np.empty(grd.size + na)
        nextgrid[0] = grd[0]
        ptr = 1
        yslices = []
        for gi, nloc in enumerate(adds):
            nextgrid[ptr : ptr + nloc + 1] = np.linspace(
                grd[gi], grd[gi + 1], 2 + nloc
            )[1:]
            nextresults[ptr + nloc] = res[gi + 1]
            nexty[ptr + nloc] = ys[gi + 1]
            if nloc > 0:
                yslices.append(slice(ptr, ptr + nloc))
            ptr += nloc + 1
        newresults = cb(np.concatenate([nextgrid[yslc] for yslc in yslices]))
        newy = (
            newresults if metric is None else np.array([metric(r) for r in newresults])
        )
        ystart, ystop = 0, 0
        for yslc in yslices:
            ystop += yslc.stop - yslc.start
            nextresults[yslc] = newresults[ystart:ystop]
            nexty[yslc] = newy[ystart:ystop]
            ystart = ystop

        return nextgrid, nextresults, nexty

    results = cb(grid)
    y = np.array(
        results if metric is None else [metric(r) for r in results], dtype=np.float64
    )

    for na in grid_additions:
        if extremum_refinement:
            extremum_cb, extremum_n, predicate_cb = extremum_refinement
            argext = extremum_cb(y)
            if predicate_cb(y, argext):
                additions = np.zeros(grid.size - 1, dtype=int)
                if argext > 0:  # left of
                    additions[argext - 1] = extremum_n
                elif argext < grid.size - 1:  # right of
                    additions[argext] = extremum_n
                grid, results, y = add_to(additions, grid, results, y)

        additions = np.zeros(grid.size - 1, dtype=int)
        done = True if atol is not None or rtol is not None else False
        slcs, errs = [], []
        for direction in ("fw", "bw"):
            est, slc = interpolate_ahead(grid, y, ntrail, direction)
            err = np.abs(y[slc] - est)
            if atol is not None:
                done = done and np.all(err < atol)
            if rtol is not None:
                done = done and np.all(err / y[slc] < rtol)
            slcs.append(slc)
            errs.append(err)

        if snr:
            all_errs = np.array(
                [[0.0] * ntrail + errs[0].tolist(), errs[1].tolist() + [0.0] * ntrail]
            )
            min__max = np.amin(all_errs, axis=0) / np.amax(all_errs, axis=0)
            dgrid = np.diff(grid)
            delta = np.empty_like(grid)
            delta[0] = dgrid[0] ** -2
            delta[-1] = dgrid[-1] ** -2
            delta[1:-1] = 1 / (dgrid[:-1] * dgrid[1:])
            lndelta = np.log(delta)
            normlndelta = lndelta - np.max(lndelta)

            for i in range(2):
                errs[i] *= (1.0 + 1e-8) - min__max[slcs[i]]
                errs[i] *= np.exp(normlndelta[slcs[i]])

        for direction, blur, slc, err in zip(("fw", "bw"), blurs, slcs, errs):
            for ib, b in enumerate(blur, 1):
                blur_slices = (slice(ib, None), slice(None, -ib))
                err[blur_slices[direction == "bw"]] += (
                    b * err[blur_slices[direction == "fw"]]
                )
            rerr = np.array(np.round(err * na / 2 / np.sum(err)), dtype=int)
            delta = np.sum(rerr) - na // 2
            if delta == 0:
                pass
            else:
                sorted_indices = np.argsort(rerr)
                for i in sorted_indices[-abs(delta) :]:
                    rerr[i] += 1 if delta < 0 else -1
                if np.sum(rerr) - na // 2:
                    raise ValueError("Balancing failed.")
            additions[
                slice(ntrail - 1, None)
                if direction == "fw"
                else slice(None, 1 - ntrail)
            ] += rerr
        grid, results, y = add_to(additions, grid, results, y)
        if done:
            break
    return grid, results



from .rebalance import (
    rebalanced_grid,
    pre_pruning_mask,
    combine_grids,
    grid_pruning_mask,
)
from .refine import refine_grid
from .make import adapted_grid
from .util import locate_discontinuity, pool_discontinuity_approx, grid_error
from .plotting import plot_convergence



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import numpy as np
from ..util import interpolate_ahead, avg_stddev


def locate_discontinuity(grid, y, consider, trnsfm=lambda x: x, ntrail=2):
    y = np.asarray(y, dtype=np.float64)
    dy = np.diff(y)
    tg = trnsfm(grid)
    dtg = np.diff(tg)
    err = np.zeros(y.size)
    for d in ("fw", "bw"):
        est, slc = interpolate_ahead(tg, y, ntrail, d)
        start = (ntrail - 1) if d == "fw" else 0
        stop = -(ntrail - 1) if d == "bw" else None
        err[slc] += np.abs(y[slc] - est) / dtg[start:stop] * dy[start:stop]
    imax = np.argsort(err)[-consider:][::-1]
    return [(tg[m], err[m]) for m in imax]


def pool_discontinuity_approx(loc_res, consistency_criterion=10):
    points = np.array(loc_res)
    w1 = np.abs(points[:, 1])
    return avg_stddev(points[:, 0], w1)


def grid_error(grid, y, ntrail=2):
    """Estimates error at each grid point from extrapolation.

    Extrapolates from left ('fw') or from right ('bw').

    Parameters
    ----------
    grid : array
    y : array
    ntrail : int
        Number of points to include in the look-ahead extrapolation.

    """
    est, _ = interpolate_ahead(grid, y, ntrail, direction="both")
    return est - y



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function

import numpy as np
from .refine import refine_grid


def adapted_grid(xstart, xstop, cb, grid_additions=(50, 50), **kwargs):
    """ " Creates an adapted (1D) grid by subsequent subgrid insertions.

    Parameters
    ----------
    xstart : float
    xstop : float
    cb : callbable
        Function to be evaluated (note that noise is handled poorly).
    grid_additions : iterable of ints (even numbers)
        Sequence specifying how many gridpoints to add each time.
    \\*\\*kwargs : see :func:`refine_grid`.
    """
    grid = np.linspace(xstart, xstop, grid_additions[0])
    return refine_grid(grid, cb, grid_additions=grid_additions[1:], **kwargs)
